@@ -42,9 +42,9 @@ struct MGL_EXPORT mglPoint
 	mglPoint(mglPoint &&d):x(d.x),y(d.y),z(d.z),c(d.c)	{}
 #endif
 	inline void Set(mreal X=0,mreal Y=0,mreal Z=0,mreal C=0)	{x=X;y=Y;z=Z;c=C;}
-	inline bool IsNAN()		{	return (x!=x || y!=y || z!=z || c!=c);	}
-	inline mreal val(int i)	{	return (i<2 ? (i==0 ? x:y) : (i==2 ? z:c));	}
-	inline mreal norm()		{	return sqrt(x*x+y*y+z*z);	}
+	inline bool IsNAN() const		{	return (x!=x || y!=y || z!=z || c!=c);	}
+	inline mreal val(int i) const	{	mreal dat[4]={x,y,z,c};	return dat[i];	}
+	inline mreal norm() const		{	return sqrt(x*x+y*y+z*z);	}
 	inline void Normalize()	{	mreal v=norm();	x/=v;	y/=v;	z/=v;	}
 
 	inline const mglPoint &operator=(const mglPoint &p)
@@ -107,6 +107,8 @@ struct MGL_EXPORT mglColor
 
 	/// Constructor for RGB components manualy
 	mglColor(float R,float G,float B, float A=1):r(R),g(G),b(B),a(A)	{}
+	/// Constructor for RGB components manualy
+	mglColor(const unsigned char *c, float A=1):r(c[0]/255.f),g(c[1]/255.f),b(c[3]/255.f),a(A)	{}
 	/// Constructor set default color
 	mglColor():r(0),g(0),b(0),a(1)	{}
 	/// Constructor set color from character id
@@ -123,17 +125,18 @@ struct MGL_EXPORT mglColor
 	{
 		if(bright<0)	bright=0;
 		if(bright>2.f)	bright=2.f;
-		r = bright<=1 ? c.r*bright : 1 - (1-c.r)*(2-bright);
-		g = bright<=1 ? c.g*bright : 1 - (1-c.g)*(2-bright);
-		b = bright<=1 ? c.b*bright : 1 - (1-c.b)*(2-bright);	a = 1;
+		if(bright<=1)
+		{	r=c.r*bright;	g=c.g*bright;	b=c.b*bright;	a = 1;	}
+		else
+		{	r=1-(1-c.r)*(2-bright);	g=1-(1-c.g)*(2-bright);	b=1-(1-c.b)*(2-bright);	a=1;	}
 	}
 	/// Check if color is valid
-	inline bool Valid()
-	{	return (r>=0 && r<=1 && g>=0 && g<=1 && b>=0 && b<=1 && a>=0 && a<=1);	}
+	inline bool Valid() const
+	{	return ((r>=0) & (r<=1) & (g>=0) & (g<=1) & (b>=0) & (b<=1) & (a>=0) & (a<=1));	}
 	/// Get maximal spectral component
-	inline float Norm()
+	inline float Norm() const
 	{	return r>g ? r : (g>b ? g : b);	}
-	inline float NormS()
+	inline float NormS() const
 	{	return r*r+g*g+b*b;	}
 	/// Set color from symbolic id
 	inline void Set(char p, float bright=1)
@@ -166,10 +169,160 @@ inline mglColor operator*(const mglColor &a, float b)
 {	return mglColor(a.r*b, a.g*b, a.b*b, a.a*b);	}
 inline mglColor operator*(float b, const mglColor &a)
 {	return mglColor(a.r*b, a.g*b, a.b*b, a.a*b);	}
+inline float operator*(const mglColor &b, const mglColor &a)
+{	return a.r*b.r+a.g*b.g+a.b*b.b;	}
 inline mglColor operator/(const mglColor &a, float b)
 {	return mglColor(a.r/b, a.g/b, a.b/b, a.a/b);	}
 inline mglColor operator!(const mglColor &a)
 {	return mglColor(1-a.r, 1-a.g, 1-a.b, a.a);	}
 #endif
 //-----------------------------------------------------------------------------
+#ifndef SWIG
+/// Class for Unicode string.
+/** NOTE: mglString accept multi-byte char* string for converting to wchar_t*. But it keep only single-byte char*!!! */
+struct MGL_EXPORT mglString
+{
+	char *s;
+	wchar_t *w;
+	mglString()	{	s=new char[1];	w=new wchar_t[1];	*s=*w=0;	}
+	mglString(const mglString &str)
+	{
+		size_t ls = wcslen(str.w)+1;
+		s = new char[ls];		memcpy(s,str.s,ls);
+		w = new wchar_t[ls];	memcpy(w,str.w,ls*sizeof(wchar_t));
+	}
+#if MGL_HAVE_RVAL
+	mglString(mglString &&d):s(d.s),w(d.w)	{	d.s=NULL;	d.w=NULL;	}
+#endif
+	mglString(const char *str)
+	{
+		if(str)
+		{
+			size_t ls=mbstowcs(0,str,0);
+			w = new wchar_t[ls+1];	mbstowcs(w,str,ls); w[ls]=0;
+			s = new char[ls+1];	for(size_t i=0;i<=ls;i++)	s[i]=w[i];
+		}
+		else	{	s=new char[1];	w=new wchar_t[1];	*s=*w=0;	}
+	}
+	mglString(const wchar_t *str)
+	{
+		if(str)
+		{
+			size_t len=wcslen(str);
+			w = new wchar_t[len+1];	s = new char[len+1];
+			for(size_t i=0;i<=len;i++)	s[i]=w[i]=str[i];
+		}
+		else	{	s=new char[1];	w=new wchar_t[1];	*s=*w=0;	}
+	}
+	mglString(const std::string &str)
+	{
+		size_t ls=mbstowcs(0,str.c_str(),0);
+		w = new wchar_t[ls+1];	mbstowcs(w,str.c_str(),ls); w[ls]=0;
+		s = new char[ls+1];	for(size_t i=0;i<=ls;i++)	s[i]=w[i];
+	}
+	mglString(const std::wstring &str)
+	{
+		size_t len=str.length();
+		w = new wchar_t[len+1];	s = new char[len+1];
+		for(size_t i=0;i<=len;i++)	s[i]=w[i]=str[i];
+	}
+	~mglString()	{	if(w)	{	delete []s;	delete []w;	}	}
+	/// String length
+	size_t length() const
+	{	return wcslen(w);	}
+	/// Crop string (like std::string::substr())
+	void crop(size_t pos, size_t len=size_t(-1))
+	{
+		if(pos)	for(size_t i=0;i<len;i++)
+			s[i] = w[i] = w[i+pos];
+		s[len] = w[len] = 0;
+	}
+	/// Find the position of symbol
+	size_t find(wchar_t ch)
+	{	const wchar_t *p = wcschr(w,ch);	return p?p-w:size_t(-1);	}
+	/// Find the position of string
+	size_t find(const wchar_t *str)
+	{	const wchar_t *p = wcsstr(w,str);	return p?p-w:size_t(-1);	}
+	size_t find(const char *str)
+	{	const char *p = strstr(s,str);		return p?p-s:size_t(-1);	}
+	/// Access to i-th symbol
+	wchar_t operator[](size_t i) const	{	return w[i];	}
+	wchar_t &operator[](size_t i)		{	return w[i];	}
+	/// Comparison operators
+	bool operator==(const char *str) const	{	return	!strcmp(s,str);	}
+	bool operator==(const wchar_t *str) const{	return	!wcscmp(w,str);	}
+	bool operator==(const std::string &str) const	{	return	str==s;	}
+	bool operator==(const std::wstring &str) const	{	return	str==w;	}
+	/// Set operators
+	const mglString &operator=(const mglString &str)
+	{
+		delete []s;	delete []w;
+		size_t ls = wcslen(str.w)+1;
+		s = new char[ls];		memcpy(s,str.s,ls);
+		w = new wchar_t[ls];	memcpy(w,str.w,ls*sizeof(wchar_t));
+		return str;
+	}
+	wchar_t operator=(wchar_t ch)
+	{
+		delete []s;	delete []w;
+		w = new wchar_t[2];	s = new char[2];
+		s[0]=w[0]=ch;	s[1]=w[1]=0;
+		return ch;
+	}
+	const std::string &operator=(const std::string &str)
+	{
+		delete []s;	delete []w;
+		size_t ls=mbstowcs(0,str.c_str(),0);
+		w = new wchar_t[ls+1];	mbstowcs(w,str.c_str(),ls); w[ls]=0;
+		s = new char[ls+1];	for(size_t i=0;i<=ls;i++)	s[i]=w[i];
+		return str;
+	}
+	const char *operator=(const char *str)
+	{
+		delete []s;	delete []w;
+		if(str)
+		{
+			size_t ls=mbstowcs(0,str,0);
+			w = new wchar_t[ls+1];	mbstowcs(w,str,ls); w[ls]=0;
+			s = new char[ls+1];	for(size_t i=0;i<=ls;i++)	s[i]=w[i];
+		}
+		else	{	s=new char[1];	w=new wchar_t[1];	*s=*w=0;	}
+		return str;
+	}
+	const std::wstring &operator=(const std::wstring &str)
+	{
+		delete []s;	delete []w;
+		size_t len=str.length();
+		w = new wchar_t[len+1];	s = new char[len+1];
+		for(size_t i=0;i<=len;i++)	s[i]=w[i]=str[i];
+		return str;
+	}
+	const wchar_t *operator=(const wchar_t *str)
+	{
+		delete []s;	delete []w;
+		if(str)
+		{
+			size_t len=wcslen(str);
+			w = new wchar_t[len+1];	s = new char[len+1];
+			for(size_t i=0;i<=len;i++)	s[i]=w[i]=str[i];
+		}
+		else	{	s=new char[1];	w=new wchar_t[1];	*s=*w=0;	}
+		return str;
+	}
+	/// Append operator
+	void operator+=(const wchar_t *str)
+	{
+		if(str && *str)
+		{
+			wchar_t *t=w;	delete []s;
+			size_t l1=wcslen(w), l2=wcslen(str);
+			w = new wchar_t[l1+l2+1];	s = new char[l1+l2+1];
+			for(size_t i=0;i<l1;i++)	s[i]=w[i]=t[i];
+			for(size_t i=0;i<=l2;i++)	s[i+l1]=w[i+l1]=str[i];
+			delete []t;
+		}
+	}
+};
+#endif
+//-----------------------------------------------------------------------------
 #endif
