@@ -143,7 +143,7 @@ Fl_Widget *add_mem(ScriptWindow *w)
 	Fl_Box *b;
 	Fl_Window *wnd = new Fl_Window(300,30,630,430,0);
 
-	b = new Fl_Box(0, 10, 630, 25, _("Existed data arrays"));	b->labeltype(FL_ENGRAVED_LABEL);
+	b = new Fl_Box(0, 10, 630, 25, _("Existing data arrays"));	b->labeltype(FL_ENGRAVED_LABEL);
 	b = new Fl_Box(0, 35, 220, 25, _("name"));
 	b->box(FL_THIN_UP_BOX);	b->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
 	b = new Fl_Box(220, 35, 205, 25, _("dimensions"));
@@ -209,14 +209,14 @@ void ScriptWindow::mem_init()
 			else if((sv>>20L)>0)	{	e=ext[3];	sv = sv>>20L;	}
 			else if((sv>>10L)>0)	{	e=ext[2];	sv = sv>>10L;	}
 			else	e=ext[1];
-			snprintf(str,128,"%ls\t%ld*%ld*%ld\t%ld %s", v->s.c_str(), v->GetNx(), v->GetNy(), v->GetNz(), sv, e);
+			snprintf(str,128,"%ls\t%ld*%ld*%ld\t%ld %s", v->Name(), v->GetNx(), v->GetNy(), v->GetNz(), sv, e);
 			var->add(str,v);
 		}
 	}
 	for(long i=0;i<Parse->GetNumConst();i++)
 	{
 		mglNum *v = Parse->GetConst(i);
-		snprintf(str,128,"%ls\t%s\t%zu b", v->s.c_str(), ("const="+mgl_str_num(v->c)).c_str(), sizeof(mglNum));
+		snprintf(str,128,"%ls\t%s\t%zu b", v->s.w, ("const="+mgl_str_num(v->c)).c_str(), sizeof(mglNum));
 		var->add(str,v);
 	}
 }
@@ -236,7 +236,7 @@ void ScriptWindow::mem_pressed(int kind)
 		w->update(v);	w->show();
 	}
 	else if(dat && kind==1)	info_dlg_cb(v);
-	else if(dat && kind==2)	Parse->DeleteVar(v->s.c_str());
+	else if(dat && kind==2)	Parse->DeleteVar(v->Name());
 	else if(kind==3)
 	{
 		const char *name = fl_input(_("Enter name for new variable"),"dat");
@@ -254,7 +254,7 @@ void ScriptWindow::mem_pressed(int kind)
 			const char *ext = fl_filename_ext(newfile);
 			if(!strcmp(ext,"h5") || !strcmp(ext,"hdf"))	// this is HDF file
 			{
-				std::string name = wcstombs(v->s);
+				std::string name = wcstombs(v->Name());
 				v->SaveHDF(newfile, name.c_str());
 			}
 			else	v->Save(newfile);
@@ -266,7 +266,7 @@ void ScriptWindow::mem_pressed(int kind)
 const char *hints[] = {
 	_("You can shift axis range by pressing middle button and moving mouse. Also, you can zoom in/out axis range by using mouse wheel."),
 	_("You can rotate/shift/zoom whole plot by mouse. Just press 'Rotate' toolbutton, click image and hold a mouse button: left button for rotation, right button for zoom/perspective, middle button for shift."),
-	_("You may quickly draw the data from file. Just use: udav 'filename.dat' in command line."),
+	_("You may quickly draw the data from file. Just use: mgllab 'filename.dat' in command line."),
 	_("You can copy the current image to clipboard by pressing Ctrl-Shift-C. Later you can paste it directly into yours document or presentation."),
 	_("You can export image into a set of format (EPS, SVG, PNG, JPEG) by pressing right mouse button inside image and selecting 'Export as ...'."),
 	_("You can setup colors for script highlighting in Property dialog. Just select menu item 'Settings/Properties'."),
@@ -409,7 +409,7 @@ public:
 		{
 			nx=dat->GetNx();	ny=dat->GetNy();	nz=dat->GetNz();
 			result = dat->PrintInfo();	out->value(result.c_str());
-			name = wcstombs(dat->s);
+			name = wcstombs(dat->Name());
 			if(nz>1)		plot_3d();
 			else if(ny>1)	plot_2d();
 			else			plot_1d();
