@@ -70,7 +70,7 @@ void mglCanvas::SetTickLen(mreal tlen, mreal stt)
 void mglCanvas::SetTicks(char dir, mreal d, int ns, mreal org, const wchar_t *lbl)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
+	mglAxis &aa = GetAxis(dir);
 
 	if(aa.f==1)	aa.t.clear();
 	aa.d=d;	aa.f=0;	aa.ns=ns;	aa.o=org;
@@ -82,8 +82,8 @@ void mglCanvas::SetTicks(char dir, mreal d, int ns, mreal org, const wchar_t *lb
 void mglCanvas::AddTick(char dir, double v, const wchar_t *lbl)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
-	bool ff = (dir=='x' ? fx : (dir=='y' ? fy : (dir=='z' ? fz : fa)));
+	mglAxis &aa = GetAxis(dir);
+	bool ff = GetFormula(dir);
 
 	UpdateAxis();	AdjustTicks(aa,ff);
 	if(!v || !lbl || !lbl[0])	{	aa.f = 0;	return;	}
@@ -97,8 +97,8 @@ void mglCanvas::AddTick(char dir, double v, const char *lbl)
 void mglCanvas::SetTicksVal(char dir, HCDT v, const wchar_t *lbl, bool add)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
-	bool ff = (dir=='x' ? fx : (dir=='y' ? fy : (dir=='z' ? fz : fa)));
+	mglAxis &aa = GetAxis(dir);
+	bool ff = GetFormula(dir);
 
 	if(add)	{	UpdateAxis();	AdjustTicks(aa,ff);	}
 	else	aa.txt.clear();
@@ -147,8 +147,8 @@ void mglCanvas::SetTicksVal(char dir, const char *lbl, bool add)
 void mglCanvas::SetTicksVal(char dir, HCDT v, const wchar_t **lbl, bool add)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
-	bool ff = (dir=='x' ? fx : (dir=='y' ? fy : (dir=='z' ? fz : fa)));
+	mglAxis &aa = GetAxis(dir);
+	bool ff = GetFormula(dir);
 
 	if(add)	{	UpdateAxis();	AdjustTicks(aa,ff);	}
 	else	aa.txt.clear();
@@ -161,8 +161,8 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const wchar_t **lbl, bool add)
 void mglCanvas::SetTicksVal(char dir, HCDT v, const char **lbl, bool add)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
-	bool ff = (dir=='x' ? fx : (dir=='y' ? fy : (dir=='z' ? fz : fa)));
+	mglAxis &aa = GetAxis(dir);
+	bool ff = GetFormula(dir);
 
 	aa.txt.clear();
 	if(add)	{	UpdateAxis();	AdjustTicks(aa,ff);	}
@@ -174,7 +174,7 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const char **lbl, bool add)
 void mglCanvas::SetTickTempl(char dir, const wchar_t *t)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
+	mglAxis &aa = GetAxis(dir);
 
 	if(aa.f==1)	aa.f = 0;	// remove time ticks
 	if(!t || !t[0])	aa.t.clear();	else aa.t=t;
@@ -183,14 +183,14 @@ void mglCanvas::SetTickTempl(char dir, const wchar_t *t)
 void mglCanvas::SetTickTempl(char dir, const char *t)
 {
 	if(!strchr("xyzca",dir))	return;
-	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
+	mglAxis &aa = GetAxis(dir);
 
 	if(aa.f==1)	aa.f = 0;	// remove time ticks
 	if(!t || !t[0])	aa.t.clear();
 	else	MGL_TO_WCS(t,aa.t=wcs);
 }
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT double mgl_adj_val(double v,mreal *ds=0)
+static double mgl_adj_val(double v,mreal *ds=0)
 {
 	double n = floor(log10(v)), s;
 	v = floor(v*pow(10.,-n));	n = pow(10.,n);
@@ -239,7 +239,7 @@ void mglCanvas::SetTickTime(char dir, mreal d, const char *t)
 			ds *= 365.25*24*3600;	}
 		// NOTE here should be months ... but it is too unregular ... so omit it now
 // 		else if(t1.tm_mon!=t2.tm_mon)	d = 30*24*3600;	// number of second in month
-		else if(abs(t1.tm_yday-t2.tm_yday)>=14)	// number of second in week 
+		else if(abs(t1.tm_yday-t2.tm_yday)>=14)	// number of second in week
 		{	d = mgl_adj_val(abs(t1.tm_yday-t2.tm_yday)/7,&ds);
 			ds = ((ds<1)?1./7:ds)*7*24*3600;	d = ((d>=1)?d:1)*7*24*3600;	}
 		else if(abs(t1.tm_yday-t2.tm_yday)>1)	// localtime("%x") cannot print time < 1 day
@@ -312,7 +312,7 @@ void mglCanvas::AdjustTicks(mglAxis &aa, bool ff)
 	LabelTicks(aa);
 }
 //-----------------------------------------------------------------------------
-int MGL_NO_EXPORT mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
+static int mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 {
 	int kind = 0;
 	if(fabs(a-b)<=0.01*fabs(a))
@@ -353,7 +353,7 @@ int MGL_NO_EXPORT mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 	return kind;
 }
 //-----------------------------------------------------------------------------
-std::wstring MGL_NO_EXPORT mgl_format(mreal v1, mreal v2, bool zero)
+/*static std::wstring mgl_format(mreal v1, mreal v2, bool zero)
 {
 	v1=fabs(v1);	v2=fabs(v2);	if(v1>v2)	v2=v1;
 	wchar_t str[5]=L"%.3g";
@@ -363,9 +363,9 @@ std::wstring MGL_NO_EXPORT mgl_format(mreal v1, mreal v2, bool zero)
 	else if(zero)
 	{	str[2] = '0'+prec;	str[3]='f';	}
 	return str;
-}
+}*/
 //-----------------------------------------------------------------------------
-std::wstring MGL_NO_EXPORT mgl_tick_text(mreal z, mreal z0, mreal d, mreal v, int kind, const std::wstring &fact, mreal step, const char *stl)
+static std::wstring mgl_tick_text(mreal z, mreal z0, mreal d, mreal v, int kind, const std::wstring &fact, mreal step, const char *stl)
 {
 	std::wstring str;
 	bool ff = step>0 && !fact.empty();// && mgl_wcslen(str)+fact.length()<62;
@@ -521,37 +521,32 @@ void mglCanvas::DrawAxis(mglAxis &aa, int text, char arr,const char *stl,mreal a
 
 	static int cgid=1;	StartGroup("Axis",cgid++);
 
-	long k1,k2;
 	bool have_color=mgl_have_color(stl);
 	bool dif_color = !have_color && aa.dv==0 && strcmp(TickStl,SubTStl);
+	long kq = AllocPnts(31);
 	if(text&2)	// line throw point (0,0,0)
 	{
 		SetPenPal("k:");
-		p = d*aa.v1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
-		for(long i=1;i<31;i++)
-		{
-			p = d*(aa.v1+(aa.v2-aa.v1)*i/30.);
-			k2 = k1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
-			line_plot(k2,k1);
-		}
+#pragma omp parallel for
+		for(long i=0;i<31;i++)
+			AddPntQ(kq+i, &B, d*(aa.v1+(aa.v2-aa.v1)*i/30.), CDef,q,-1,3);
+		curve_plot(31,kq);
 	}
 	SetPenPal(have_color ? stl:AxisStl);
 
-	p = o + d*aa.v1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
-	for(long i=1;i<31;i++)	// axis itself
-	{
-		p = o + d*(aa.v1+(aa.v2-aa.v1)*i/30.);
-		k2 = k1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
-		line_plot(k2,k1);
-	}
+	kq = AllocPnts(31);
+#pragma omp parallel for
+	for(long i=0;i<31;i++)
+		AddPntQ(kq+i, &B, o + d*(aa.v1+(aa.v2-aa.v1)*i/30.), CDef,q,-1,3);
+	curve_plot(31,kq);
 	if(arr)
 	{
 		p = o + d*(aa.v1+(aa.v2-aa.v1)*1.05);
-		k2 = k1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
-		line_plot(k1,k2);	arrow_plot(k1,k2,arr);
+		long k1 = AddPnt(&B, p,CDef,q,-1,3);
+		line_plot(k1,kq+30);	arrow_plot(k1,kq+30,arr);
 	}
 
-	k2 = aa.txt.size();
+	long k2 = aa.txt.size();
 	mreal v, u, v0 = mgl_isnan(aa.o) ? aa.v0 : aa.o;
 	if(*TickStl && !have_color)	SetPenPal(TickStl);
 	if(k2>0)	for(long i=0;i<k2;i++)
@@ -598,12 +593,14 @@ void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 
 	long n = aa.txt.size();
 	mreal *w=new mreal[n], wsp = 2*TextWidth(" ",FontDef,-1);
-	long *kk=new long[n];
+	long *kk=new long[n], kq = AllocPnts(n);
+#pragma omp parallel for
 	for(long i=0;i<n;i++)	// fill base label properties
 	{
 		w[i] = TextWidth(aa.txt[i].text.c_str(),FontDef,-1);
-		kk[i] = AddPnt(M, o+d*aa.txt[i].val,-1,d,0,7);
+		AddPntQ(kq+i, M, o+d*aa.txt[i].val,-1,d,0,7);
 	}
+	for(long i=0;i<n;i++)	kk[i] = kq+i;
 	mreal c=INFINITY, l=0, h = TextHeight(FontDef,-1);	// find sizes
 	for(long i=0;i<n-1;i++)
 	{
@@ -666,7 +663,7 @@ void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 		if(get(MGL_NO_ORIGIN) && v==aa.v0)	continue;
 		if(v>aa.v1 && v<aa.v2 && i%k!=0)	continue;
 		char pos[4]={up[i],':',align[i],0};
-		text_plot(kk[i], aa.txt[i].text.c_str(), pos, -1, aa.sh+0.1,CDef);
+		text_plot(kk[i], aa.txt[i].text.c_str(), pos, -1, aa.sh+0.05,CDef);
 	}
 	delete []w;	delete []kk;	delete []align;	delete []up;
 }
@@ -705,8 +702,8 @@ void mglCanvas::tick_draw(mglPoint o, mglPoint d1, mglPoint d2, int f)
 
 	ScalePoint(&B,o, d1, false);	d1.Normalize();
 	ScalePoint(&B,p, d2, false);	d2.Normalize();
-	long k2 = AddPnt(&B, p, CDef, mglPoint(NAN), 0, 0);
 	long k1 = AddPnt(&B, p+d1*v, CDef, mglPoint(NAN), 0, 0);
+	long k2 = AddPnt(&B, p, CDef, mglPoint(NAN), 0, 0);
 	long k3 = AddPnt(&B, p+d2*v, CDef, mglPoint(NAN), 0, 0);
 	line_plot(k1,k2);	line_plot(k2,k3);
 }
@@ -726,28 +723,20 @@ void mglCanvas::Grid(const char *dir, const char *pen, const char *opt)
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_drw_grid(HMGL gr, double val, const mglPoint &d, const mglPoint &oa, const mglPoint &ob, const mglPoint &da1, const mglPoint &db1, const mglPoint &da2, const mglPoint &db2)
+static void mgl_drw_grid(HMGL gr, double val, const mglPoint &d, const mglPoint &oa, const mglPoint &ob, const mglPoint &da1, const mglPoint &db1, const mglPoint &da2, const mglPoint &db2)
 {
-	gr->Reserve(62);
-	mglPoint q,p;
-	q = oa+d*val;	p = q+da1;	// lines along 'a'
-	long k1 = gr->AddPnt(p), k2;
-	for(long j=1;j<31;j++)
-	{
-		mreal v = j/30.;
-		p = q+da1*(1-v)+da2*v;
-		k2 = k1;	k1 = gr->AddPnt(p);
-		gr->line_plot(k2,k1);
-	}
-	q = ob+d*val;	p = q+db1;	// lines along 'b'
-	k1 = gr->AddPnt(p);
-	for(long j=1;j<31;j++)
-	{
-		mreal v = j/30.;
-		p = q+db1*(1-v)+db2*v;
-		k2 = k1;	k1 = gr->AddPnt(p);
-		gr->line_plot(k2,k1);
-	}
+	mglPoint q(oa+d*val);	// lines along 'a'
+	long kq = gr->AllocPnts(31);
+#pragma omp parallel for
+	for(long i=0;i<31;i++)
+	{	mreal v=i/30.;	gr->AddPntQ(kq+i,q+da1*(1-v)+da2*v);	}
+	gr->curve_plot(31,kq);
+	q = ob+d*val;		// lines along 'b'
+	kq = gr->AllocPnts(31);
+#pragma omp parallel for
+	for(long i=0;i<31;i++)
+	{	mreal v = i/30.;	gr->AddPntQ(kq+i,q+db1*(1-v)+db2*v);	}
+	gr->curve_plot(31,kq);
 }
 void mglCanvas::DrawGrid(mglAxis &aa, bool at_tick)
 {
@@ -806,6 +795,7 @@ void mglCanvas::Label(char dir, const char *str, mreal pos, const char *opt)
 void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt)
 {
 	mreal shift =  SaveState(opt), t=0;	if(mgl_isnan(shift))	shift=0;
+	shift -= 0.1;
 	mglPoint p,q;
 	mglAxis *aa=0;
 
@@ -943,27 +933,24 @@ void mglCanvas::Box(const char *col, bool ticks)
 					else	{	color[0]=col[i];	color[1]=0;	break;	}
 				}
 			}
-			long *pos = new long[3*31*31];
 			SetPenPal(color);
 			mreal dx = (Max.x-Min.x)/30, dy = (Max.y-Min.y)/30, dz = (Max.z-Min.z)/30;
+			long kq = AllocPnts(3*31*31);
+#pragma omp parallel for collapse(2)
 			for(long i=0;i<31;i++)	for(long j=0;j<31;j++)
 			{
-				long i0=3*(i+31*j);
-				pos[i0]   = AddPnt(mglPoint(oo[im].x,Min.y+dy*i,Min.z+dz*j));
-				pos[i0+1] = AddPnt(mglPoint(Min.x+dx*i,oo[im].y,Min.z+dz*j));
-				pos[i0+2] = AddPnt(mglPoint(Min.x+dx*i,Min.y+dy*j,oo[im].z));
+				long i0=kq+3*(i+31*j);
+				AddPntQ(i0,  mglPoint(oo[im].x,Min.y+dy*i,Min.z+dz*j));
+				AddPntQ(i0+1,mglPoint(Min.x+dx*i,oo[im].y,Min.z+dz*j));
+				AddPntQ(i0+2,mglPoint(Min.x+dx*i,Min.y+dy*j,oo[im].z));
 			}
 			for(long i=0;i<30;i++)	for(long j=0;j<30;j++)
 			{
-				long i0=3*(i+31*j);
-				quad_plot(pos[i0],pos[i0+3],pos[i0+93],pos[i0+96]);
-				quad_plot(pos[i0+1],pos[i0+4],pos[i0+94],pos[i0+97]);
-				quad_plot(pos[i0+2],pos[i0+5],pos[i0+95],pos[i0+98]);
+				long i0=kq+3*(i+31*j);
+				quad_plot(i0,  i0+3,i0+93,i0+96);
+				quad_plot(i0+1,i0+4,i0+94,i0+97);
+				quad_plot(i0+2,i0+5,i0+95,i0+98);
 			}
-			delete []pos;
-// 			mgl_facex(this, oo[im].x, Min.y, Min.z, Max.y-Min.y, Max.z-Min.z, color,0,0);
-// 			mgl_facey(this, Min.x, oo[im].y, Min.z, Max.x-Min.x, Max.z-Min.z, color,0,0);
-// 			mgl_facez(this, Min.x, Min.y, oo[im].z, Max.x-Min.x, Max.y-Min.y, color,0,0);
 		}
 	}
 	EndGroup();
@@ -1120,7 +1107,11 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 			case 3:	p1.y = y;	p2.y = y+0.1*h;	break;
 			default:p1.x = x-0.1*w;	p2.x = x;	break;
 		}
-		line_plot(AddPnt(&M, p1), AddPnt(&M, p2));
+		mglPoint p3(0.75*p1.x+0.25*p2.x, 0.75*p1.y+0.25*p2.y, s3);
+		mglPoint p4(0.25*p1.x+0.75*p2.x, 0.25*p1.y+0.75*p2.y, s3);
+//		line_plot(AddPnt(&M, p1), AddPnt(&M, p2));
+		line_plot(AddPnt(&M, p1), AddPnt(&M, p3));
+		line_plot(AddPnt(&M, p4), AddPnt(&M, p2));
 	}
 	ac.dir.Set(ss*w,ss*h,0);	ac.a.Set(0,0,0);
 	ac.org.Set(w+x,h+y,s3+1);	ac.b.Set(0,0,0);
